local function getModPath()
    local info
    local i = 1
    repeat
        info = debug.getinfo(i,'S')
        i = i+1
    until info == nil

    local modPath = ""
    if i > 1 then
        info = debug.getinfo(i-2,'S')
        if info then
            if string.ends(info.source, 'mod.lua') then
                modPath = string.gsub(info.source, "@(.*/)mod[.]lua", "%1")
            elseif string.ends(info.source, '.mdl') then
                modPath = string.gsub(info.source, "@(.*/)res/models/model/.+[.]mdl", "%1")
            elseif string.ends(info.source, '.con') then
                modPath = string.gsub(info.source, "@(.*/)res/construction/.+[.]con", "%1")
            elseif string.ends(info.source, '.msh') then
                modPath = string.gsub(info.source, "@(.*/)res/models/mesh/.+[.]msh", "%1")
            elseif string.ends(info.source, '.grp') then
                modPath = string.gsub(info.source, "@(.*/)res/models/group/.+[.]grp", "%1")
            elseif string.ends(info.source, '.mtl') then
                modPath = string.gsub(info.source, "@(.*/)res/models/material/.+[.]mtl", "%1")
            elseif string.ends(info.source, '.ani') then
                modPath = string.gsub(info.source, "@(.*/)res/models/animation/.+[.]ani", "%1")
            elseif string.find(info.source, "/workshop/content/1066780/") then
                modPath = string.gsub(info.source, "@(.*/workshop/content/1066780/[^/]*/).*", "%1")
            elseif string.find(info.source, "/staging_area/") then
                modPath = string.gsub(info.source, "@(.*/staging_area/[^/]*/).*", "%1")
            elseif string.find(info.source, "/mods/") then
                modPath = string.gsub(info.source, "@(.*/mods/[^/]*/).*", "%1")
            end
        end
    end

    return modPath
end

local mnutil = {}

function mnutil.escapeRegex(text)
    return text:gsub("([^%w])", "%%%1")
end

function mnutil.pretty(obj, max_depth)
    if max_depth == nil then max_depth = 1 end

    -- Returns true if a table has a __tostring metamethod.
    local function coerceable(tbl)
        local meta = getmetatable(tbl)
        return (meta and meta.__tostring)
    end

    local depth = 1
    local indent = ""
    local function recurse(obj)
        if type(obj) == "string" then
            -- Dump the string so that escape sequences are printed.
            return string.format("%q", obj)
        elseif type(obj) == "table" and not coerceable(obj) then
            if pairs(obj)(obj) == nil then
                return '{}' -- Always print empty tables.
            end
            if depth > max_depth then
                return tostring(obj)
            end

            local str = indent.."{"
            local old_indent = indent
            depth = depth + 1
            indent = indent .. "  "

            for k, v in pairs(obj) do
                local pair = mnutil.pretty(k, 0).." = "..recurse(v)
                str = str..(str == old_indent.."{" and "\n" or ",\n")..indent..pair
            end

            depth = depth - 1
            indent = old_indent
            return str.."\n"..indent.."}"
        else
            -- tostring() can fail if there is an error in a __tostring metamethod.
            local success, value = pcall(function() return tostring(obj) end)
            return (success and value or "<!!error in __tostring metamethod!!>")
        end
    end

    return recurse(obj)
end

mnutil.getModPath = getModPath()

return mnutil