local vec3 = require "vec3"
local transf = require "transf"
local mnutil = require "mnutil"

local function makeTrack(result, transform, tag, slotId, addModuleFn, params, edgeListNum, origMakeTrackFn)
    origMakeTrackFn(result, transform, tag, slotId, addModuleFn, params, edgeListNum, origMakeTrackFn)

    local function GetTerminals(i, j, leftOrRight)
        if leftOrRight then return result.trackCoord2models[i][j].left
        else return result.trackCoord2models[i][j].right end
    end

    local function AddTerminal(i, j, leftOrRight, modelNum)
        if leftOrRight then result.trackCoord2models[i][j].left = modelNum
        else result.trackCoord2models[i][j].right =  modelNum end
    end

    local function numberOfCargoPlatforms(i, j, direction)
        local platformCount = 0
        local nextM
        repeat
            platformCount = platformCount + 1
            nextM = result.GetModuleAt(i+platformCount*2*direction, j)
        until not nextM or not nextM.metadata.cargo_platform
        return platformCount * ((nextM and nextM.metadata.track and 1) or 2)
    end

    --store assignment information between cargo platform and terminal (for preventing assigment more terminals to one platform)
    local function assignCargoTerminal(i, j)
        result.cargoTerminalsAssigned[j] = result.cargoTerminalsAssigned[j] or {}
        if (result.cargoTerminalsAssigned[j] and result.cargoTerminalsAssigned[j][i]) then
            print ("Enhanced train station capacity: Multiple cargo terminal assignment at i: " .. i .. ", j: " .. j)
            return false
        end

        result.cargoTerminalsAssigned[j][i] = result.terminalID
        return true
    end

    local coords = result.GetCoord(slotId)
    local i = coords[1]
    local j = coords[2]

    result.cargoTerminalsAssigned = result.cargoTerminalsAssigned or {}
    result.terminalID = result.terminalID or 0

    local cargoPlatformsInfo = {
        [-1] = {count = {[0] = 0}, maxj = {[-1] = false, [1] = false}, trackPos = {[-1] = false, [1] = false}},
        [1] = {count = {[0] = 0}, maxj = {[-1] = false, [1] = false}, trackPos = {[-1] = false, [1] = false}},
        middle = {[0]=false}
    }

    local tmpModule
    for inci=1,-1,-2 do
        tmpModule = result.GetModuleAt(i+inci, j)
        if tmpModule and tmpModule.metadata.cargo_platform then
            cargoPlatformsInfo[inci].count[0] = numberOfCargoPlatforms(i+inci, j, inci)
        end
    end

    if (cargoPlatformsInfo[-1].count[0] or cargoPlatformsInfo[1].count[0]) then
        --find all cargo platforms on next/previous slot in the direction of the track, stop when there is another rail in particular "platform column"
        for incj=1,-1,-2 do  -- next/previous relative to the direction of the track
            local stop = {[-1] = cargoPlatformsInfo[-1].count[0] == 0, [1] = cargoPlatformsInfo[1].count[0] == 0}
            local dj=0 -- delta j relative to the track coord
            repeat
                local inTheMiddle = false
                dj = dj+incj
                tmpModule = result.GetModuleAt(i, j+dj)
                if tmpModule and tmpModule.metadata.track then --test if there is a track in the middle
                    local trackPos = math.abs(dj)
                    for inci=1,-1,-2 do
                        cargoPlatformsInfo[inci].trackPos[incj] = cargoPlatformsInfo[inci].trackPos[incj] or trackPos
                        cargoPlatformsInfo[inci].maxj[incj] = cargoPlatformsInfo[inci].maxj[incj] or trackPos-1
                        stop[inci] = true
                    end
                    break
                end
                inTheMiddle = tmpModule and tmpModule.metadata.cargo_platform
                cargoPlatformsInfo.middle[dj] = inTheMiddle

                local mindi=0
                local maxdi=0
                local wasTrack=false
                for inci=1,-1,-2 do  -- forward/backward direction (=platforms places)
                    local di=0  -- delta i relative to the track coord
                    if not stop[inci] or inTheMiddle then
                        repeat
                            mindi = math.min(mindi, di)
                            maxdi = math.max(maxdi, di)
                            di = di + inci
                            tmpModule = result.GetModuleAt(i+di, j+dj)
                            if tmpModule and tmpModule.metadata.track then
                                local trackPos = math.abs(dj)
                                cargoPlatformsInfo[inci].trackPos[incj] = trackPos
                                cargoPlatformsInfo[inci].maxj[incj] = cargoPlatformsInfo[inci].maxj[incj] or trackPos-1
                                stop[inci] = true
                                wasTrack = true
                            end
                        until not tmpModule or not tmpModule.metadata.cargo_platform
                        if di==inci then
                            stop[inci] = true
                            cargoPlatformsInfo[inci].maxj[incj] = cargoPlatformsInfo[inci].maxj[incj] or math.abs(dj)-1
                        end
                    end
                end
                if wasTrack and inTheMiddle then
                    --mark track also at other side if there is valid cargo platform
                    local trackPos = math.abs(dj)
                    for inci=1,-1,-2 do
                        if not stop[inci] then
                            cargoPlatformsInfo[inci].trackPos[incj] = trackPos
                            cargoPlatformsInfo[inci].maxj[incj] = trackPos-1
                            stop[inci] = true
                        end
                    end
                end
                cargoPlatformsInfo[-1].count[dj] = math.abs(mindi)
                cargoPlatformsInfo[1].count[dj] = maxdi
            until stop[-1] and stop[1]

            --find any tracks next slots for preventing overlaping terminals from other tracks
            for inci = 1, -1, -2 do
                if (cargoPlatformsInfo[inci].maxj[incj] or 0) > 0 then
                    local mindi = 0
                    local maxdi = 0

                    dj = 0
                    stop = false

                    repeat
                        dj = dj + incj
                        local trackPos = math.abs(dj)
                        if (cargoPlatformsInfo[inci].trackPos[incj] or 0) == trackPos then break end

                        maxdi = math.max(maxdi, cargoPlatformsInfo[inci].count[dj] or 0)
                        local minPlatf = false
                        local maxPlatf = false

                        for adi=mindi-1,maxdi+1 do
                            tmpModule = result.GetModuleAt(i+adi*inci, j+dj)
                            if tmpModule then
                                if tmpModule.metadata.track then
                                    cargoPlatformsInfo[inci].trackPos[incj] = trackPos
                                    stop = true
                                    break
                                end
                                if tmpModule.metadata.cargo_platform and adi>=mindi and adi<=maxdi then
                                    minPlatf = minPlatf or adi
                                    maxPlatf = adi
                                end
                            end
                        end
                        if minPlatf == false then break end

                        mindi = minPlatf
                        maxdi = maxPlatf
                    until stop
                end
            end
        end

        local addedTerminals = {[0] = {}}
        local tresh = 0

        --add terminal definitions to the found platforms
        for inci=1,-1,-2 do --forward/backward terminal
            local terminals = GetTerminals(i, j, inci == -1) or {}
            if (#terminals>1) then
                print ("Enhanced train station capacity: Conflict with other mods when adding more cargo platforms to the terminal - there is more then one cargo platform within terminal")
                break
            end
            local djLimits = { [1] = cargoPlatformsInfo[inci].trackPos[1] and math.ceil(cargoPlatformsInfo[inci].trackPos[1] / 2) or true,
                               [-1] = cargoPlatformsInfo[inci].trackPos[-1] and math.ceil((cargoPlatformsInfo[inci].trackPos[-1]+1) / 2) or true }
            local maxCount = cargoPlatformsInfo[inci].count[0];
            local addMiddle = inci==1;
            for di=#terminals,maxCount-1 do --if there is terminal from original function, we will skip adding first cargo platform
                if assignCargoTerminal(i+(di+1)*inci, j) then
                    if inci == 1 then
                        addModuleFn("station/rail/era_a/tn_cargo_wait_area.mdl", transf.rotZTransl(math.rad(180), vec3.new(3.5+di*5, 0, 0)))
                    else
                        addModuleFn("station/rail/era_a/tn_cargo_wait_area.mdl", transf.transl(vec3.new(-3.5-di*5, 0, 0)))
                    end
                    terminals[#terminals+1] = {#result.models - 1, 0}
                    addedTerminals[0][#addedTerminals[0]+1] = (di+1)*inci
                end
            end

            for incj=1,-1,-2 do
                local dj = incj
                maxCount = math.min(cargoPlatformsInfo[inci].count[0], cargoPlatformsInfo[inci].count[dj] or 0);
                while maxCount>0 and (djLimits[incj]==true or (djLimits[incj]>math.abs(dj))) do
                    addedTerminals[dj] = addedTerminals[dj] or {}
                    addMiddle = addMiddle and cargoPlatformsInfo.middle[dj]
                    if addMiddle then
                        if assignCargoTerminal(i, j+dj) then
                            addedTerminals[dj][#addedTerminals[dj]+1] = 0
                            addModuleFn("station/rail/era_a/tn_cargo_wait_area.mdl", transf.transl(vec3.new(0, dj*40, 0)))
                            terminals[#terminals+1] = {#result.models - 1, 0}
                        end
                    end
                    for di=0,maxCount-1 do
                        if assignCargoTerminal(i+(di+1)*inci, j+dj) then
                            if inci == 1 then
                                addModuleFn("station/rail/era_a/tn_cargo_wait_area.mdl", transf.rotZTransl(math.rad(180), vec3.new(3.5+di*5, dj*40, 0)))
                            else
                                addModuleFn("station/rail/era_a/tn_cargo_wait_area.mdl", transf.transl(vec3.new(-3.5-di*5, dj*40, 0)))
                            end
                            terminals[#terminals+1] = {#result.models - 1, 0}
                            addedTerminals[dj][#addedTerminals[dj]+1] = (di+1)*inci
                        end
                    end
                    dj = dj + incj
                    maxCount = math.min(maxCount, cargoPlatformsInfo[inci].count[dj] or 0);
                end
            end

            if (#terminals > 0) then
                AddTerminal(i, j, inci==-1, terminals)
                result.terminalID = result.terminalID + 1
            end
        end
    end

end

local enhcapacityhelper = {}

function enhcapacityhelper.onLoadTrainStationUtil(data)
    local origMakeTrackFn = data.makeTrack
    data.makeTrack = function (result, transform, tag, slotId, addModuleFn, params, edgeListNum)
        makeTrack(result, transform, tag, slotId, addModuleFn, params, edgeListNum, origMakeTrackFn)
    end
    return data
end

return enhcapacityhelper