local modifiers = {}

local function loadPackage(packageName)
    local origPreload = package.preload[packageName]
    package.preload[packageName] = nil

    data = require(packageName)
    for i, v in ipairs(modifiers[packageName] or {}) do
        data = v(data)
    end
    package.preload[packageName] = origPreload

    return data
end

local packageModifier = {}
function packageModifier.addModifier(packageName, callback)
    if package.loaded[packageName] ~= nil then
        print ("PackageModifier: Force reload package:"..packageName.."'\n")
        package.loaded[packageName] = nil
    end
    modifiers[packageName] = modifiers[packageName] or {}
    table.insert(modifiers[packageName], callback)
    if (#modifiers[packageName]==1) then
        if (package.preload[packageName] ~= nil) then
            print ("PackageModifier: Conflict in package preloader for package '"..packageName.."'\n")
            return
        end
        package.preload[packageName] = function()
            return loadPackage(packageName)
        end
    end
end

return packageModifier